@@ -12,14 +12,15 @@ Copyright 2008 Heikki Toivonen. All rights reserved.
 
 import logging
 import socket
+import io
 
-from M2Crypto import BIO, Err, X509, m2, six, util  # noqa
+from M2Crypto import BIO, Err, X509, m2, py27plus, six, util  # noqa
 from M2Crypto.SSL import Checker, Context, timeout  # noqa
 from M2Crypto.SSL import SSLError
 from M2Crypto.SSL.Cipher import Cipher, Cipher_Stack
 from M2Crypto.SSL.Session import Session
-if util.py27plus:
-    from typing import Any, AnyStr, Callable, Dict, List, Optional, Tuple, Union  # noqa
+if py27plus:
+    from typing import Any, AnyStr, Callable, Optional, Tuple, Union  # noqa
 
 __all__ = ['Connection',
            'timeout',  # XXX Not really, but for documentation purposes
@@ -33,14 +34,14 @@ def _serverPostConnectionCheck(*args, **kw):
     return 1
 
 
-class Connection:
+class Connection(object):
     """An SSL connection."""
 
-    clientPostConnectionCheck = Checker.Checker()
     serverPostConnectionCheck = _serverPostConnectionCheck
 
     m2_bio_free = m2.bio_free
     m2_ssl_free = m2.ssl_free
+    m2_bio_noclose = m2.bio_noclose
 
     def __init__(self, ctx, sock=None, family=socket.AF_INET):
         # type: (Context, socket.socket, int) -> None
@@ -50,6 +51,11 @@ class Connection:
         :param sock: socket to be used
         :param family: socket family
         """
+        # The Checker needs to be an instance attribute
+        # and not a class attribute for thread safety reason
+        self.clientPostConnectionCheck = Checker.Checker()
+
+        self._bio_freed = False
         self.ctx = ctx
         self.ssl = m2.ssl_new(self.ctx.ctx)  # type: bytes
         if sock is not None:
@@ -71,23 +77,38 @@ class Connection:
 
         self.host = None
 
+    def _free_bio(self):
+        """
+           Free the sslbio and sockbio, and close the socket.
+        """
+        # Do not do it twice
+        if not self._bio_freed:
+            if getattr(self, 'sslbio', None):
+                self.m2_bio_free(self.sslbio)
+            if getattr(self, 'sockbio', None):
+                self.m2_bio_free(self.sockbio)
+            if self.ssl_close_flag == self.m2_bio_noclose and \
+                    getattr(self, 'ssl', None):
+                self.m2_ssl_free(self.ssl)
+            self.socket.close()
+            self._bio_freed = True
+
+
     def __del__(self):
         # type: () -> None
-        if getattr(self, 'sslbio', None):
-            self.m2_bio_free(self.sslbio)
-        if getattr(self, 'sockbio', None):
-            self.m2_bio_free(self.sockbio)
-        # in __del__ method we have to check whether m2.bio_noclose
-        # exists at all.
-        if m2 is not None and m2.bio_noclose and \
-                self.ssl_close_flag == m2.bio_noclose and \
-                getattr(self, 'ssl', None):
-            self.m2_ssl_free(self.ssl)
-        self.socket.close()
+        # Notice that M2Crypto doesn't automatically shuts down the
+        # connection here. You have to call self.close() in your
+        # program, M2Crypto won't do it automatically for you.
+        self._free_bio()
 
-    def close(self):
+    def close(self, freeBio=False):
+        """
+           if freeBio is true, call _free_bio
+        """
         # type: () -> None
         m2.ssl_shutdown(self.ssl)
+        if freeBio:
+            self._free_bio()
 
     def clear(self):
         # type: () -> int
@@ -386,7 +407,7 @@ class Connection:
         buflen = len(buff_bytes)
 
         # memoryview type has been added in 2.7
-        if util.py27plus and isinstance(buff, memoryview):
+        if py27plus and isinstance(buff, memoryview):
             buff[:buflen] = buff_bytes
             buff[buflen:] = b'\x00' * (len(buff) - buflen)
         else:
@@ -573,7 +594,7 @@ class Connection:
     def get_cipher_list(self, idx=0):
         # type: (int) -> str
         """Return the cipher suites for this connection as a string object."""
-        return util.py3str(m2.ssl_get_cipher_list(self.ssl, idx))
+        return six.ensure_text(m2.ssl_get_cipher_list(self.ssl, idx))
 
     def set_cipher_list(self, cipher_list):
         # type: (str) -> int
@@ -581,9 +602,12 @@ class Connection:
         return m2.ssl_set_cipher_list(self.ssl, cipher_list)
 
     def makefile(self, mode='rb', bufsize=-1):
-        # type: (AnyStr, int) -> socket._fileobject
+        # type: (AnyStr, int) -> Union[io.BufferedRWPair,io.BufferedReader]
         if six.PY3:
-            return socket.SocketIO(self, mode)
+            raw = socket.SocketIO(self, mode)
+            if 'rw' in mode:
+                return io.BufferedRWPair(raw, raw)
+            return io.BufferedReader(raw, io.DEFAULT_BUFFER_SIZE)
         else:
             return socket._fileobject(self, mode, bufsize)
 
@@ -636,11 +660,20 @@ class Connection:
             self.socket.getsockopt(socket.SOL_SOCKET, socket.SO_RCVTIMEO,
                                    timeout.struct_size()))
 
+    @staticmethod
+    def _hexdump(s):
+        assert isinstance(s, six.binary_type)
+        return ":".join("{0:02x}".format(ord(c) if six.PY2 else c) for c in s)
+
     def get_socket_write_timeout(self):
         # type: () -> timeout
-        return timeout.struct_to_timeout(
-            self.socket.getsockopt(socket.SOL_SOCKET, socket.SO_SNDTIMEO,
-                                   timeout.struct_size()))
+        binstr = self.socket.getsockopt(
+            socket.SOL_SOCKET, socket.SO_SNDTIMEO, timeout.struct_size())
+        timeo = timeout.struct_to_timeout(binstr)
+        # print("Debug: get_socket_write_timeout: "
+        #       "get sockopt value: %s -> ret timeout(sec=%r, microsec=%r)" %
+        #       (self._hexdump(binstr), timeo.sec, timeo.microsec))
+        return timeo
 
     def set_socket_read_timeout(self, timeo):
         # type: (timeout) -> None
@@ -651,13 +684,17 @@ class Connection:
     def set_socket_write_timeout(self, timeo):
         # type: (timeout) -> None
         assert isinstance(timeo, timeout.timeout)
+        binstr = timeo.pack()
+        # print("Debug: set_socket_write_timeout: "
+        #       "input timeout(sec=%r, microsec=%r) -> set sockopt value: %s" %
+        #       (timeo.sec, timeo.microsec, self._hexdump(binstr)))
         self.socket.setsockopt(
-            socket.SOL_SOCKET, socket.SO_SNDTIMEO, timeo.pack())
+            socket.SOL_SOCKET, socket.SO_SNDTIMEO, binstr)
 
     def get_version(self):
         # type: () -> str
         """Return the TLS/SSL protocol version for this connection."""
-        return util.py3str(m2.ssl_get_version(self.ssl))
+        return six.ensure_text(m2.ssl_get_version(self.ssl))
 
     def set_post_connection_check_callback(self, postConnectionCheck):  # noqa
         # type: (Callable) -> None
