@@ -12,8 +12,8 @@ Author: Heikki Toivonen
 import binascii
 import logging
 
-from M2Crypto import ASN1, BIO, EVP, Err, m2, util  # noqa
-if util.py27plus:
+from M2Crypto import ASN1, BIO, EVP, m2, py27plus, six  # noqa
+if py27plus:
     from typing import AnyStr, Optional  # noqa
 
 FORMAT_DER = 0
@@ -41,7 +41,7 @@ def new_extension(name, value, critical=0, _pyfree=1):
     Create new X509_Extension instance.
     """
     if name == 'subjectKeyIdentifier' and \
-            value.strip('0123456789abcdefABCDEF:') is not '':
+            value.strip('0123456789abcdefABCDEF:') != '':
         raise ValueError('value must be precomputed hash')
     ctx = m2.x509v3_set_nconf()
     x509_ext_ptr = m2.x509v3_ext_conf(None, ctx, name, value)
@@ -54,7 +54,7 @@ def new_extension(name, value, critical=0, _pyfree=1):
     return x509_ext
 
 
-class X509_Extension:  # noqa
+class X509_Extension(object):
     """
     X509 Extension
     """
@@ -102,7 +102,7 @@ class X509_Extension:  # noqa
         """
         Get the extension name, for example 'subjectAltName'.
         """
-        return util.py3str(m2.x509_extension_get_name(self.x509_ext))
+        return six.ensure_text(m2.x509_extension_get_name(self.x509_ext))
 
     def get_value(self, flag=0, indent=0):
         # type: (int, int) -> str
@@ -114,10 +114,10 @@ class X509_Extension:  # noqa
         """
         buf = BIO.MemoryBuffer()
         m2.x509_ext_print(buf.bio_ptr(), self.x509_ext, flag, indent)
-        return util.py3str(buf.read_all())
+        return six.ensure_text(buf.read_all())
 
 
-class X509_Extension_Stack:  # noqa
+class X509_Extension_Stack(object):
     """
     X509 Extension Stack
 
@@ -195,7 +195,7 @@ class X509_Extension_Stack:  # noqa
         return self.pystack.pop()
 
 
-class X509_Name_Entry:  # noqa
+class X509_Name_Entry(object):
     """
     X509 Name Entry
     """
@@ -256,7 +256,7 @@ class X509_Name_Entry:  # noqa
                                                 field, type, entry, len)
 
 
-class X509_Name:  # noqa
+class X509_Name(object):
     """
     X509 Name
     """
@@ -314,7 +314,7 @@ class X509_Name:  # noqa
         if attr in self.nid:
             assert m2.x509_name_type_check(self.x509_name), \
                 "'x509_name' type error"
-            return util.py3str(m2.x509_name_by_nid(self.x509_name, self.nid[attr]))
+            return six.ensure_text(m2.x509_name_by_nid(self.x509_name, self.nid[attr]))
 
         if attr in self.__dict__:
             return self.__dict__[attr]
@@ -330,7 +330,7 @@ class X509_Name:  # noqa
             assert m2.x509_name_type_check(self.x509_name), \
                 "'x509_name' type error"
             return m2.x509_name_set_by_nid(self.x509_name, self.nid[attr],
-                                           util.py3bytes(value))
+                                           six.ensure_binary(value))
 
         self.__dict__[attr] = value
 
@@ -383,8 +383,8 @@ class X509_Name:  # noqa
         :return: 1 for success of 0 if an error occurred.
         """
         return m2.x509_name_add_entry_by_txt(self.x509_name,
-                                             util.py3str(field), type,
-                                             entry, len, loc, set)
+                                             six.ensure_str(field), type,
+                                             six.ensure_str(entry), len, loc, set)
 
     def entry_count(self):
         # type: () -> int
@@ -425,7 +425,7 @@ class X509_Name:  # noqa
             "'x509_name' type error"
         buf = BIO.MemoryBuffer()
         m2.x509_name_print_ex(buf.bio_ptr(), self.x509_name, indent, flags)
-        return util.py3str(buf.read_all())
+        return six.ensure_text(buf.read_all())
 
     def as_der(self):
         # type: () -> bytes
@@ -440,7 +440,7 @@ class X509_Name:  # noqa
         return m2.x509_name_hash(self.x509_name)
 
 
-class X509:
+class X509(object):
     """
     X.509 Certificate
     """
@@ -477,7 +477,7 @@ class X509:
         assert m2.x509_type_check(self.x509), "'x509' type error"
         buf = BIO.MemoryBuffer()
         m2.x509_print(buf.bio_ptr(), self.x509)
-        return util.py3str(buf.read_all())
+        return six.ensure_text(buf.read_all())
 
     def as_der(self):
         # type: () -> bytes
@@ -689,7 +689,7 @@ class X509:
         m2x509_extension_get_name = m2.x509_extension_get_name
         x509 = self.x509
 
-        name = util.py3bytes(name)
+        name = six.ensure_binary(name)
         for i in range(m2.x509_get_ext_count(x509)):
             ext_ptr = m2x509_get_ext(x509, i)
             if m2x509_extension_get_name(ext_ptr) == name:
@@ -783,7 +783,7 @@ class X509:
         md = EVP.MessageDigest(md)
         md.update(der)
         digest = md.final()
-        return util.py3str(binascii.hexlify(digest).upper())
+        return six.ensure_text(binascii.hexlify(digest).upper())
 
 
 def load_cert(file, format=FORMAT_PEM):
@@ -846,7 +846,7 @@ def load_cert_string(string, format=FORMAT_PEM):
 
     :return: M2Crypto.X509.X509 object.
     """
-    string = util.py3bytes(string)
+    string = six.ensure_binary(string)
     bio = BIO.MemoryBuffer(string)
     return load_cert_bio(bio, format)
 
@@ -860,13 +860,13 @@ def load_cert_der_string(string):
 
     :return: M2Crypto.X509.X509 object.
     """
-    string = util.py3bytes(string)
+    string = six.ensure_binary(string)
     bio = BIO.MemoryBuffer(string)
     cptr = m2.d2i_x509(bio._ptr())
     return X509(cptr, _pyfree=1)
 
 
-class X509_Store_Context:  # noqa
+class X509_Store_Context(object):
     """
     X509 Store Context
     """
@@ -930,7 +930,7 @@ class X509_Store_Context:  # noqa
         return X509_Stack(m2.x509_store_ctx_get1_chain(self.ctx), 1, 1)
 
 
-class X509_Store:  # noqa
+class X509_Store(object):
     """
     X509 Store
     """
@@ -1003,8 +1003,17 @@ class X509_Store:  # noqa
 
     add_cert = add_x509
 
+    def set_flags(self, flags):
+        """
+        Set the verification flags for the X509Store
+        Wrapper over OpenSSL X509_STORE_set_flags()
 
-class X509_Stack:  # noqa
+        :param flags: verification parameters
+        """
+        return m2.x509_store_set_flags(self.store, flags)
+
+
+class X509_Stack(object):
     """
     X509 Stack
 
@@ -1097,12 +1106,12 @@ def new_stack_from_der(der_string):
 
     :return: X509_Stack
     """
-    der_string = util.py3bytes(der_string)
+    der_string = six.ensure_binary(der_string)
     stack_ptr = m2.make_stack_from_der_sequence(der_string)
     return X509_Stack(stack_ptr, 1, 1)
 
 
-class Request:
+class Request(object):
     """
     X509 Certificate Request.
     """
@@ -1128,7 +1137,7 @@ class Request:
         # type: () -> str
         buf = BIO.MemoryBuffer()
         m2.x509_req_print(buf.bio_ptr(), self.req)
-        return util.py3str(buf.read_all())
+        return six.ensure_text(buf.read_all())
 
     def as_pem(self):
         # type: () -> bytes
@@ -1320,7 +1329,7 @@ def load_request_string(string, format=FORMAT_PEM):
 
     :return: M2Crypto.X509.Request object.
     """
-    string = util.py3bytes(string)
+    string = six.ensure_binary(string)
     bio = BIO.MemoryBuffer(string)
     return load_request_bio(bio, format)
 
@@ -1333,12 +1342,12 @@ def load_request_der_string(string):
     :param string: String containing a certificate request in DER format.
     :return: M2Crypto.X509.Request object.
     """
-    string = util.py3bytes(string)
+    string = six.ensure_binary(string)
     bio = BIO.MemoryBuffer(string)
     return load_request_bio(bio, FORMAT_DER)
 
 
-class CRL:
+class CRL(object):
     """
     X509 Certificate Revocation List
     """
@@ -1373,7 +1382,7 @@ class CRL:
         """
         buf = BIO.MemoryBuffer()
         m2.x509_crl_print(buf.bio_ptr(), self.crl)
-        return util.py3str(buf.read_all())
+        return six.ensure_text(buf.read_all())
 
 
 def load_crl(file):
